@@ -9,6 +9,7 @@
 #include "abstractmetalang.h"
 #include "abstractmetatype.h"
 #include "documentation.h"
+#include "exception.h"
 #include "modifications.h"
 #include "messages.h"
 #include "propertyspec.h"
@@ -17,11 +18,14 @@
 #include "complextypeentry.h"
 #include "functiontypeentry.h"
 #include "enumtypeentry.h"
+#include "typesystemtypeentry.h"
+#include "typedatabase.h"
 
 #include "qtcompat.h"
 
 #include <QtCore/QDir>
 #include <QtCore/QFile>
+#include <QtCore/QHash>
 #include <QtCore/QUrl>
 
 using namespace Qt::StringLiterals;
@@ -37,6 +41,60 @@ Documentation QtDocParser::retrieveModuleDocumentation()
     return retrieveModuleDocumentation(packageName());
 }
 
+// Return the package of a type "PySide6.QtGui.QPainter" -> "PySide6.QtGui"
+static QStringView packageFromPythonType(QStringView pythonType)
+{
+    qsizetype pos = pythonType.startsWith(u"PySide6.") ? 8 : 0;
+    auto dot = pythonType.indexOf(u'.', pos);
+    return dot != -1 ? pythonType.sliced(0, dot) : pythonType;
+}
+
+// Return the qdoc dir "PySide6.QtGui.QPainter" -> "qtgui/webxml" (QTBUG-119500)
+static QString qdocModuleDirFromPackage(QStringView package)
+{
+    if (package.startsWith(u"PySide6."))
+        package = package.sliced(8);
+    return package.toString().toLower() + "/webxml"_L1;
+}
+
+// Populate a cache of package to WebXML dir
+static QHash<QString, QString> getPackageToModuleDir()
+{
+    QHash<QString, QString> result;
+    const auto &typeSystemEntries = TypeDatabase::instance()->typeSystemEntries();
+    for (const auto &te : typeSystemEntries) {
+        const QString &package = te->name();
+        const QString &docPackage = te->hasDocTargetLangPackage()
+                                    ? te->docTargetLangPackage() : package;
+        result.insert(package, qdocModuleDirFromPackage(docPackage));
+    }
+    return result;
+}
+
+QString QtDocParser::qdocModuleDir(const QString &pythonType)
+{
+    static const QHash<QString, QString> packageToModuleDir = getPackageToModuleDir();
+
+    const QStringView package = packageFromPythonType(pythonType);
+    const auto it = packageToModuleDir.constFind(package);
+    if (it == packageToModuleDir.cend()) {
+        const QString known = packageToModuleDir.keys().join(", "_L1);
+        qCWarning(lcShibokenDoc, "Type from unknown package: \"%s\" (known: %s).",
+                  qPrintable(pythonType), qPrintable(known));
+        return qdocModuleDirFromPackage(package);
+    }
+    return it.value();
+}
+
+static QString xmlFileNameRoot(const AbstractMetaClassPtr &metaClass)
+{
+    QString className = metaClass->qualifiedCppName().toLower();
+    className.replace("::"_L1, "-"_L1);
+
+    return QtDocParser::qdocModuleDir(metaClass->typeEntry()->targetLangPackage())
+           + u'/' + className;
+}
+
 static void formatPreQualifications(QTextStream &str, const AbstractMetaType &type)
 {
     if (type.isConstant())
@@ -63,7 +121,7 @@ static void formatFunctionUnqualifiedArgTypeQuery(QTextStream &str,
         const auto flagsEntry = std::static_pointer_cast<const FlagsTypeEntry>(metaType.typeEntry());
         QString name = flagsEntry->qualifiedCppName();
         if (name.endsWith(u'>') && name.startsWith(u"QFlags<")) {
-            const int lastColon = name.lastIndexOf(u':');
+            const auto lastColon = name.lastIndexOf(u':');
             if (lastColon != -1) {
                 name.replace(lastColon + 1, name.size() - lastColon - 1, metaType.name());
                 name.remove(0, 7);
@@ -76,7 +134,7 @@ static void formatFunctionUnqualifiedArgTypeQuery(QTextStream &str,
         break;
     case AbstractMetaType::ContainerPattern: { // QVector<int>
         str << metaType.typeEntry()->qualifiedCppName() << '<';
-        const auto instantiations = metaType.instantiations();
+        const auto &instantiations = metaType.instantiations();
         for (qsizetype i = 0, size = instantiations.size(); i < size; ++i) {
             if (i)
                 str << ", ";
@@ -208,12 +266,15 @@ static QString extractBrief(QString *value)
 // Find the webxml file for global functions/enums
 // by the doc-file typesystem attribute or via include file.
 static QString findGlobalWebXmLFile(const QString &documentationDataDirectory,
+                                    const QString &package,
                                     const QString &docFile,
                                     const Include &include)
 {
     QString result;
+    const QString root = documentationDataDirectory + u'/'
+                         + QtDocParser::qdocModuleDir(package) + u'/';
     if (!docFile.isEmpty()) {
-        result = documentationDataDirectory + u'/' + docFile;
+        result = root + docFile;
         if (!result.endsWith(webxmlSuffix))
             result += webxmlSuffix;
         return QFileInfo::exists(result) ? result : QString{};
@@ -221,8 +282,7 @@ static QString findGlobalWebXmLFile(const QString &documentationDataDirectory,
     if (include.name().isEmpty())
         return {};
     // qdoc "\headerfile <QtLogging>" directive produces "qtlogging.webxml"
-    result = documentationDataDirectory + u'/' +
-             QFileInfo(include.name()).baseName() + webxmlSuffix;
+    result = root + QFileInfo(include.name()).baseName() + webxmlSuffix;
     if (QFileInfo::exists(result))
         return result;
     // qdoc "\headerfile <qdrawutil.h>" produces "qdrawutil-h.webxml"
@@ -237,7 +297,7 @@ void  QtDocParser::fillGlobalFunctionDocumentation(const AbstractMetaFunctionPtr
         return;
 
     const QString sourceFileName =
-        findGlobalWebXmLFile(documentationDataDirectory(), te->docFile(), te->include());
+        findGlobalWebXmLFile(documentationDataDirectory(), te->targetLangPackage(), te->docFile(), te->include());
     if (sourceFileName.isEmpty())
         return;
 
@@ -252,7 +312,7 @@ void  QtDocParser::fillGlobalFunctionDocumentation(const AbstractMetaFunctionPtr
                               {}, f, &errorMessage);
     if (!errorMessage.isEmpty())
         qCWarning(lcShibokenDoc, "%s", qPrintable(errorMessage));
-    const Documentation documentation(detailed, {});
+    Documentation documentation(detailed, {}, sourceFileName);
     f->setDocumentation(documentation);
 }
 
@@ -260,7 +320,7 @@ void QtDocParser::fillGlobalEnumDocumentation(AbstractMetaEnum &e)
 {
     auto te = e.typeEntry();
     const QString sourceFileName =
-        findGlobalWebXmLFile(documentationDataDirectory(), te->docFile(), te->include());
+        findGlobalWebXmLFile(documentationDataDirectory(), te->targetLangPackage(), te->docFile(), te->include());
     if (sourceFileName.isEmpty())
         return;
 
@@ -270,16 +330,16 @@ void QtDocParser::fillGlobalEnumDocumentation(AbstractMetaEnum &e)
         qCWarning(lcShibokenDoc, "%s", qPrintable(errorMessage));
         return;
     }
-    if (!extractEnumDocumentation(classDocumentationO.value(), e)) {
+    if (!extractEnumDocumentation(classDocumentationO.value(), sourceFileName, e)) {
         qCWarning(lcShibokenDoc, "%s",
                   qPrintable(msgCannotFindDocumentation(sourceFileName, {}, e, {})));
     }
 }
 
-void QtDocParser::fillDocumentation(const AbstractMetaClassPtr &metaClass)
+QString QtDocParser::fillDocumentation(const AbstractMetaClassPtr &metaClass)
 {
     if (!metaClass)
-        return;
+        return {};
 
     auto context = metaClass->enclosingClass();
     while (context) {
@@ -288,9 +348,7 @@ void QtDocParser::fillDocumentation(const AbstractMetaClassPtr &metaClass)
         context = context->enclosingClass();
     }
 
-    QString sourceFileRoot = documentationDataDirectory() + u'/'
-        + metaClass->qualifiedCppName().toLower();
-    sourceFileRoot.replace(u"::"_s, u"-"_s);
+    QString sourceFileRoot = documentationDataDirectory() + u'/' + xmlFileNameRoot(metaClass);
 
     QFileInfo sourceFile(sourceFileRoot + webxmlSuffix);
     if (!sourceFile.exists())
@@ -299,7 +357,7 @@ void QtDocParser::fillDocumentation(const AbstractMetaClassPtr &metaClass)
         qCWarning(lcShibokenDoc).noquote().nospace()
             << "Can't find qdoc file for class " << metaClass->name() << ", tried: "
             << QDir::toNativeSeparators(sourceFile.absoluteFilePath());
-        return;
+       return {};
     }
 
     const QString sourceFileName = sourceFile.absoluteFilePath();
@@ -308,12 +366,12 @@ void QtDocParser::fillDocumentation(const AbstractMetaClassPtr &metaClass)
     const auto classDocumentationO = parseWebXml(sourceFileName, &errorMessage);
     if (!classDocumentationO.has_value()) {
         qCWarning(lcShibokenDoc, "%s", qPrintable(errorMessage));
-        return;
+        return {};
     }
 
     const auto &classDocumentation = classDocumentationO.value();
     for (const auto &p : classDocumentation.properties) {
-        Documentation doc(p.description, p.brief);
+        Documentation doc(p.description, p.brief, sourceFileName);
         metaClass->setPropertyDocumentation(p.name, doc);
     }
 
@@ -328,6 +386,7 @@ void QtDocParser::fillDocumentation(const AbstractMetaClassPtr &metaClass)
     const QString brief = extractBrief(&docString);
 
     Documentation doc;
+    doc.setSourceFile(sourceFileName);
     if (!brief.isEmpty())
         doc.setValue(brief, Documentation::Brief);
     doc.setValue(docString);
@@ -341,7 +400,7 @@ void QtDocParser::fillDocumentation(const AbstractMetaClassPtr &metaClass)
                                   metaClass, func, &errorMessage);
         if (!errorMessage.isEmpty())
             qCWarning(lcShibokenDoc, "%s", qPrintable(errorMessage));
-        const Documentation documentation(detailed, {});
+        const Documentation documentation(detailed, {}, sourceFileName);
         std::const_pointer_cast<AbstractMetaFunction>(func)->setDocumentation(documentation);
     }
 #if 0
@@ -358,17 +417,19 @@ void QtDocParser::fillDocumentation(const AbstractMetaClassPtr &metaClass)
 #endif
     // Enums
     for (AbstractMetaEnum &meta_enum : metaClass->enums()) {
-        if (!extractEnumDocumentation(classDocumentation, meta_enum)) {
+        if (!extractEnumDocumentation(classDocumentation, sourceFileName, meta_enum)) {
             qCWarning(lcShibokenDoc, "%s",
                       qPrintable(msgCannotFindDocumentation(sourceFileName, metaClass, meta_enum, {})));
         }
     }
+
+    return sourceFileName;
 }
 
 bool QtDocParser::extractEnumDocumentation(const ClassDocumentation &classDocumentation,
+                                           const QString &sourceFileName,
                                            AbstractMetaEnum &meta_enum)
 {
-    Documentation enumDoc;
     const auto index = classDocumentation.indexOfEnum(meta_enum.name());
     if (index == -1)
         return false;
@@ -382,7 +443,7 @@ bool QtDocParser::extractEnumDocumentation(const ClassDocumentation &classDocume
             doc.insert(firstPara + 6, note);
         }
     }
-    enumDoc.setValue(doc);
+    Documentation enumDoc(doc, {}, sourceFileName);
     meta_enum.setDocumentation(enumDoc);
     return true;
 }
@@ -401,19 +462,20 @@ Documentation QtDocParser::retrieveModuleDocumentation(const QString& name)
 {
     // TODO: This method of acquiring the module name supposes that the target language uses
     // dots as module separators in package names. Improve this.
-    QString moduleName = name;
-    moduleName.remove(0, name.lastIndexOf(u'.') + 1);
-    if (moduleName == u"QtQuickControls2")
-        moduleName.chop(1);
-    const QString prefix = documentationDataDirectory() + u'/'
-        + moduleName.toLower();
+    QString completeModuleName = name;
+    if (completeModuleName.endsWith("QtQuickControls2"_L1))
+        completeModuleName.chop(1);
+    const QString moduleName = completeModuleName.sliced(name.lastIndexOf(u'.') + 1);
+    const QString lowerModuleName = moduleName.toLower();
 
-    const QString sourceFile = prefix + u"-index.webxml"_s;
+    const QString prefix = documentationDataDirectory() + u'/'
+                           + qdocModuleDir(completeModuleName) + u'/' + lowerModuleName;
+    const QString sourceFile = prefix + "-index.webxml"_L1;
     if (!QFile::exists(sourceFile)) {
         qCWarning(lcShibokenDoc).noquote().nospace()
             << "Can't find qdoc file for module " <<  name << ", tried: "
             << QDir::toNativeSeparators(sourceFile);
-        return Documentation();
+        return {};
     }
 
     QString errorMessage;
@@ -423,7 +485,7 @@ Documentation QtDocParser::retrieveModuleDocumentation(const QString& name)
         return {};
     }
 
-    Documentation doc(docString, {});
+    Documentation doc(docString, {}, sourceFile);
     if (doc.isEmpty()) {
         qCWarning(lcShibokenDoc, "%s",
                   qPrintable(msgCannotFindDocumentation(sourceFile, "module", name)));
@@ -431,7 +493,7 @@ Documentation QtDocParser::retrieveModuleDocumentation(const QString& name)
     }
 
     // If a QML module info file exists, insert a link to the Qt docs.
-    const QFileInfo qmlModuleFi(prefix + u"-qmlmodule.webxml"_s);
+    const QFileInfo qmlModuleFi(prefix + "-qmlmodule.webxml"_L1);
     if (qmlModuleFi.isFile()) {
         QString docString = doc.detailed();
         const int pos = docString.lastIndexOf(u"</description>");
