@@ -29,6 +29,7 @@ public:
         Default, // "{}"
         Zero,
         MinusOne,
+        NullPtr,
         Void
     };
 
@@ -45,17 +46,26 @@ public:
 
 protected:
     QString fileNameForContext(const GeneratorContext &context) const override;
-    void generateClass(TextStream &s, const GeneratorContext &classContext) override;
+    void generateClass(TextStream &s, const QString &targetDir,
+                       const GeneratorContext &classContext) override;
     bool finishGeneration() override;
 
 private:
+    struct VirtualMethodReturn
+    {
+        QString statement;
+        bool needsReference = false;
+    };
+
+
     void generateSmartPointerClass(TextStream &s, const GeneratorContext &classContext);
     void generateIncludes(TextStream &s, const GeneratorContext &classContext,
                           const IncludeGroupList &includes = {},
                           const AbstractMetaClassCList &innerClasses = {}) const;
-    static void writeInitFunc(TextStream &declStr, TextStream &callStr,
-                              const QString &initFunctionName,
-                              const TypeEntryCPtr &enclosingEntry = {});
+    static void writeInitFuncCall(TextStream &callStr,
+                                  const QString &functionName,
+                                  const TypeEntryCPtr &enclosingEntry,
+                                  const QString &pythonName, bool lazy = true);
     static void writeCacheResetNative(TextStream &s, const GeneratorContext &classContext);
     void writeConstructorNative(TextStream &s, const GeneratorContext &classContext,
                                 const AbstractMetaFunctionCPtr &func) const;
@@ -75,13 +85,20 @@ private:
     void writeVirtualMethodNative(TextStream &s,
                                   const AbstractMetaFunctionCPtr &func,
                                   int cacheIndex) const;
+    void writeVirtualMethodPythonOverride(TextStream &s,
+                                          const AbstractMetaFunctionCPtr &func,
+                                          const CodeSnipList &snips,
+                                          const VirtualMethodReturn &returnStatement) const;
+    void writeUserAddedPythonOverride(TextStream &s,
+                                      const AbstractMetaFunctionCPtr &func) const;
     void writeVirtualMethodCppCall(TextStream &s, const AbstractMetaFunctionCPtr &func,
                                    const QString &funcName, const QList<CodeSnip> &snips,
                                    const AbstractMetaArgument *lastArg, const TypeEntryCPtr &retType,
                                    const QString &returnStatement, bool hasGil) const;
-    static QString virtualMethodReturn(TextStream &s, const ApiExtractorResult &api,
-                                       const AbstractMetaFunctionCPtr &func,
-                                       const FunctionModificationList &functionModifications);
+
+    static VirtualMethodReturn virtualMethodReturn(const ApiExtractorResult &api,
+                                                   const AbstractMetaFunctionCPtr &func,
+                                                   const FunctionModificationList &functionModifications);
     void writeMetaObjectMethod(TextStream &s, const GeneratorContext &classContext) const;
     static void writeMetaCast(TextStream &s, const GeneratorContext &classContext);
 
@@ -106,6 +123,7 @@ private:
         QString converterCheckFunctionName;
         QString pythonToConverterFunctionName;
         QString registrationCode;
+        bool hasQVariantConversion = false;
     };
 
     OpaqueContainerData
@@ -133,6 +151,7 @@ private:
                             const AbstractMetaFunctionCList &overloads,
                             const GeneratorContext &classContext) const;
     static void writeArgumentsInitializer(TextStream &s, const OverloadData &overloadData,
+                                          const GeneratorContext &classContext,
                                           ErrorReturn errorReturn = ErrorReturn::Default);
     static void writeCppSelfConversion(TextStream &s,
                                        const GeneratorContext &context,
@@ -161,6 +180,7 @@ private:
                                   ErrorReturn errorReturn);
 
     static QString returnErrorWrongArguments(const OverloadData &overloadData,
+                                             const GeneratorContext &context,
                                              ErrorReturn errorReturn);
 
     static void writeFunctionReturnErrorCheckSection(TextStream &s,
@@ -179,6 +199,8 @@ private:
     static void writeTypeCheck(TextStream& s, const std::shared_ptr<OverloadDataNode> &overloadData,
                         const QString &argumentName);
 
+    static void replacePolymorphicIdPlaceHolders(const AbstractMetaClassCPtr &metaClass,
+                                                 QString *id);
     static void writeTypeDiscoveryFunction(TextStream &s,
                                            const AbstractMetaClassCPtr &metaClass);
 
@@ -268,6 +290,7 @@ private:
      *   \param overloadData the overload data describing all the possible overloads for the function/method
      */
     void writeOverloadedFunctionDecisor(TextStream &s, const OverloadData &overloadData,
+                                        const GeneratorContext &classContext,
                                         ErrorReturn errorReturn) const;
     /// Recursive auxiliar method to the other writeOverloadedFunctionDecisor.
     void writeOverloadedFunctionDecisorEngine(TextStream &s,
@@ -354,6 +377,7 @@ private:
                                              const AbstractMetaFunctionCPtr &func,
                                              bool usePyArgs,
                                              const OverloadData &overloadData,
+                                             const GeneratorContext &classContext,
                                              ErrorReturn errorReturn);
 
     /// Returns a string containing the name of an argument for the given function and argument index.
@@ -378,10 +402,12 @@ private:
     static void writeSignatureStrings(TextStream &s, const QString &signatures,
                                       const QString &arrayName,
                                       const char *comment);
+    void writeInitInheritance(TextStream &s) const;
     void writeClassRegister(TextStream &s,
                             const AbstractMetaClassCPtr &metaClass,
                             const GeneratorContext &classContext,
                             const QString &signatures) const;
+    static QStringList pyBaseTypes(const AbstractMetaClassCPtr &metaClass);
     static QString destructorClassName(const AbstractMetaClassCPtr &metaClass,
                                        const GeneratorContext &classContext);
     static void writeStaticFieldInitialization(TextStream &s,
@@ -393,7 +419,8 @@ private:
     QList<PyMethodDefEntry> methodDefinitionEntries(const OverloadData &overloadData) const;
 
     void writeSignatureInfo(TextStream &s, const OverloadData &overloads) const;
-    QString signatureParameter(const AbstractMetaArgument &arg) const;
+    QString signatureParameter(const AbstractMetaArgument &arg, bool implicitConversions) const;
+    QString pythonSignature(const AbstractMetaType &type) const;
     /// Writes the implementation of all methods part of python sequence protocol
     void writeSequenceMethods(TextStream &s,
                               const AbstractMetaClassCPtr &metaClass,
@@ -459,8 +486,12 @@ private:
                                                       const CustomConversionPtr &customConversion);
     static void writeEnumConverterInitialization(TextStream &s, const AbstractMetaEnum &metaEnum);
     static QString writeContainerConverterInitialization(TextStream &s,
-                                                         const AbstractMetaType &type);
+                                                         const AbstractMetaType &type,
+                                                         const ApiExtractorResult &api);
     void writeSmartPointerConverterInitialization(TextStream &s, const AbstractMetaType &ype) const;
+
+    static QString typeInitStruct(const TypeEntryCPtr &te);
+    static QString typeInitStruct(const GeneratorContext &context);
     static void writeExtendedConverterInitialization(TextStream &s,
                                                      const TypeEntryCPtr &externalType,
                                                      const AbstractMetaClassCList &conversions);
@@ -504,8 +535,7 @@ private:
     /// Write default implementations for sequence protocol
     void writeDefaultSequenceMethods(TextStream &s, const GeneratorContext &context) const;
     /// Helper function for writeStdListWrapperMethods.
-    static void writeIndexError(TextStream &s, const QString &errorMsg,
-                                ErrorReturn errorReturn);
+    static void writeIndexError(TextStream &s, ErrorReturn errorReturn);
 
     static QString writeReprFunctionHeader(TextStream &s, const GeneratorContext &context);
     static QString writeReprFunction(TextStream &s,
@@ -528,12 +558,11 @@ private:
     static bool hasBoolCast(const AbstractMetaClassCPtr &metaClass)
     { return boolCast(metaClass).has_value(); }
 
-    std::optional<AbstractMetaType>
-        findSmartPointerInstantiation(const SmartPointerTypeEntryCPtr &pointer,
-                                      const TypeEntryCPtr &pointee) const;
     void clearTpFuncs();
     static QString chopType(QString s);
 
+    static QString typeInitStructHelper(const TypeEntryCPtr &te, const QString &varName);
+
     QHash<QString, QString> m_tpFuncs;
     QHash<QString, QString> m_nbFuncs;
 };
