@@ -105,13 +105,13 @@ static PyType_Spec typeprobe_spec = {
 static void
 check_PyTypeObject_valid()
 {
-    auto *obtype = reinterpret_cast<PyObject *>(&PyType_Type);
-    auto *probe_tp_base = reinterpret_cast<PyTypeObject *>(
-        PyObject_GetAttr(obtype, Shiboken::PyMagicName::base()));
+    auto *typetype = &PyType_Type;
+    auto *obtype = reinterpret_cast<PyObject *>(typetype);
+    auto *probe_tp_base_obj = PyObject_GetAttr(obtype, Shiboken::PyMagicName::base());
+    auto *probe_tp_base = reinterpret_cast<PyTypeObject *>(probe_tp_base_obj);
     auto *probe_tp_bases = PyObject_GetAttr(obtype, Shiboken::PyMagicName::bases());
-    auto *check = reinterpret_cast<PyTypeObject *>(
-        PyType_FromSpecWithBases(&typeprobe_spec, probe_tp_bases));
-    auto *typetype = reinterpret_cast<PyTypeObject *>(obtype);
+    auto *checkObj = PyType_FromSpecWithBases(&typeprobe_spec, probe_tp_bases);
+    auto *check = reinterpret_cast<PyTypeObject *>(checkObj);
     PyObject *w = PyObject_GetAttr(obtype, Shiboken::PyMagicName::weakrefoffset());
     long probe_tp_weakrefoffset = PyLong_AsLong(w);
     PyObject *d = PyObject_GetAttr(obtype, Shiboken::PyMagicName::dictoffset());
@@ -149,8 +149,8 @@ check_PyTypeObject_valid()
         || probe_tp_mro             != typetype->tp_mro
         || Py_TPFLAGS_DEFAULT       != (check->tp_flags & Py_TPFLAGS_DEFAULT))
         Py_FatalError("The structure of type objects has changed!");
-    Py_DECREF(check);
-    Py_DECREF(probe_tp_base);
+    Py_DECREF(checkObj);
+    Py_DECREF(probe_tp_base_obj);
     Py_DECREF(w);
     Py_DECREF(d);
     Py_DECREF(probe_tp_bases);
@@ -482,6 +482,47 @@ Pep_GetVerboseFlag()
 }
 #endif // Py_LIMITED_API
 
+// Support for pyerrors.h
+
+#if defined(Py_LIMITED_API) || PY_VERSION_HEX < 0x030C0000
+// Emulate PyErr_GetRaisedException() using the deprecated PyErr_Fetch()/PyErr_Store()
+PyObject *PepErr_GetRaisedException()
+{
+    PyObject *type{};
+    PyObject *value{};
+    PyObject *traceback{};
+    PyErr_Fetch(&type, &value, &traceback);
+    Py_XINCREF(value);
+    PyErr_Restore(type, value, traceback);
+    return value;
+}
+
+struct PepException_HEAD
+{
+    PyObject_HEAD
+    PyObject *x1; // dict
+    PyObject *args;
+};
+
+// PyException_GetArgs/PyException_SetArgs were added to the stable API in 3.12
+PyObject *PepException_GetArgs(PyObject *ex)
+{
+    auto *h = reinterpret_cast<PepException_HEAD *>(ex);
+    Py_XINCREF(h->args);
+    return h->args;
+}
+
+LIBSHIBOKEN_API void PepException_SetArgs(PyObject *ex, PyObject *args)
+{
+    auto *h = reinterpret_cast<PepException_HEAD *>(ex);
+    Py_XINCREF(args);
+    auto *old = h->args; // Py_XSETREF()
+    h->args = args;
+    Py_XDECREF(old);
+
+}
+#endif // Limited or < 3.12
+
 /*****************************************************************************
  *
  * Support for code.h
@@ -855,15 +896,13 @@ PepRun_GetResult(const char *command)
     /*
      * Evaluate a string and return the variable `result`
      */
-    PyObject *d, *v, *res;
-
-    d = PyDict_New();
+    PyObject *d = PyDict_New();
     if (d == nullptr
         || PyDict_SetItem(d, Shiboken::PyMagicName::builtins(), PyEval_GetBuiltins()) < 0) {
         return nullptr;
     }
-    v = PyRun_String(command, Py_file_input, d, d);
-    res = v ? PyDict_GetItem(d, Shiboken::PyName::result()) : nullptr;
+    PyObject *v = PyRun_String(command, Py_file_input, d, d);
+    PyObject *res = v ? PyDict_GetItem(d, Shiboken::PyName::result()) : nullptr;
     Py_XDECREF(v);
     Py_DECREF(d);
     return res;
@@ -871,7 +910,7 @@ PepRun_GetResult(const char *command)
 
 PyTypeObject *PepType_Type_tp_new(PyTypeObject *metatype, PyObject *args, PyObject *kwds)
 {
-    auto ret = PyType_Type.tp_new(metatype, args, kwds);
+    auto *ret = PyType_Type.tp_new(metatype, args, kwds);
     return reinterpret_cast<PyTypeObject *>(ret);
 }
 
@@ -900,7 +939,7 @@ _Pep_PrivateMangle(PyObject *self, PyObject *name)
         Py_INCREF(name);
         return name;
     }
-    size_t nlen = PyUnicode_GET_LENGTH(name);
+    const Py_ssize_t nlen = PyUnicode_GET_LENGTH(name);
     /* Don't mangle __id__ or names with dots. */
     if ((PyUnicode_READ_CHAR(name, nlen-1) == '_' &&
          PyUnicode_READ_CHAR(name, nlen-2) == '_') ||
@@ -914,9 +953,9 @@ _Pep_PrivateMangle(PyObject *self, PyObject *name)
     // PYSIDE-1436: _Py_Mangle is no longer exposed; implement it always.
     // The rest of this function is our own implementation of _Py_Mangle.
     // Please compare the original function in compile.c .
-    size_t plen = PyUnicode_GET_LENGTH(privateobj.object());
+    Py_ssize_t plen = PyUnicode_GET_LENGTH(privateobj.object());
     /* Strip leading underscores from class name */
-    size_t ipriv = 0;
+    Py_ssize_t ipriv = 0;
     while (PyUnicode_READ_CHAR(privateobj.object(), ipriv) == '_')
         ipriv++;
     if (ipriv == plen) {
@@ -930,8 +969,8 @@ _Pep_PrivateMangle(PyObject *self, PyObject *name)
                         "private identifier too large to be mangled");
         return nullptr;
     }
-    size_t const amount = ipriv + 1 + plen + nlen;
-    size_t const big_stack = 1000;
+    const Py_ssize_t amount = ipriv + 1 + plen + nlen;
+    const Py_ssize_t big_stack = 1000;
     wchar_t bigbuf[big_stack];
     wchar_t *resbuf = amount <= big_stack ? bigbuf : (wchar_t *)malloc(sizeof(wchar_t) * amount);
     if (!resbuf)
@@ -954,19 +993,6 @@ _Pep_PrivateMangle(PyObject *self, PyObject *name)
  *
  */
 
-int PepRuntime_38_flag = 0;
-
-static void
-init_PepRuntime()
-{
-    // We expect a string of the form "\d\.\d+\."
-    const char *version = Py_GetVersion();
-    if (version[0] < '3')
-        return;
-    if (std::atoi(version + 2) >= 8)
-        PepRuntime_38_flag = 1;
-}
-
 static long _GetPepRuntimeVersion()
 {
     auto *version = PySys_GetObject("version_info");
@@ -1106,10 +1132,8 @@ SbkEnumTypePrivate *PepType_SETP(SbkEnumType *enumType)
     if (enumType == SETP_key)
         return SETP_value;
     auto it = SETP_extender.find(enumType);
-    if (it == SETP_extender.end()) {
-        it = SETP_extender.insert({enumType, {}}).first;
-        memset(&it->second, 0, sizeof(SbkEnumTypePrivate));
-    }
+    if (it == SETP_extender.end())
+        it = SETP_extender.insert({enumType, SbkEnumTypePrivate{nullptr, nullptr}}).first;
     SETP_key = enumType;
     SETP_value = &it->second;
     return SETP_value;
@@ -1262,7 +1286,6 @@ static inline void *PepType_ExTP(PyTypeObject *type, size_t size)
 void
 Pep384_Init()
 {
-    init_PepRuntime();
 #ifdef Py_LIMITED_API
     check_PyTypeObject_valid();
     Pep_GetVerboseFlag();
@@ -1276,3 +1299,17 @@ Pep384_Init()
 }
 
 } // extern "C"
+
+LIBSHIBOKEN_API bool PepExt_Weakref_IsAlive(PyObject *weakRef)
+{
+#if defined(Py_LIMITED_API) || PY_VERSION_HEX < 0x030D0000
+    return PyWeakref_GetObject(weakRef) != Py_None;
+#else
+    // FIXME: Make this the default code path once Limited API has been raised to 3.13
+    // Note: PyWeakref_GetObject() will be removed in 3.15.
+    PyObject *pobj = nullptr;
+    const bool result = PyWeakref_GetRef(weakRef, &pobj) == 1;
+    Py_XDECREF(pobj);
+    return result;
+#endif
+}
